from .base import Base

class Brand(Base):
	endpoint = '/brand'

	def get_brand_image(self):
		return self.client.get(
			self.endpoint + '/image'
		)

	def upload_brand_image(self, data=None):
		return self.client.post(
			self.endpoint + '/image',
			data=data
		)



import json
import ssl
import asyncio
import logging
import websockets

logging.basicConfig(level=logging.INFO)
log = logging.getLogger('mattermostdriver.websocket')


class Websocket:
	def __init__(self, options, token):
		self.options = options
		self._token = token

	@asyncio.coroutine
	def connect(self, event_handler):
		"""
		Connect to the websocket and authenticate it.
		When the authentication has finished, start the loop listening for messages,
		sending a ping to the server to keep the connection alive.
		:param event_handler: Every websocket event will be passed there
		:type event_handler: Function
		:return:
		"""
		context = ssl.create_default_context(purpose=ssl.Purpose.CLIENT_AUTH)
		if not self.options['verify']:
			context.verify_mode = ssl.CERT_NONE

		scheme = 'wss://'
		if self.options['scheme'] != 'https':
			scheme = 'ws://'

		url = scheme + self.options['url'] + ':' + str(self.options['port']) + self.options['basepath'] + '/websocket'

		websocket = yield from websockets.connect(
			url,
			ssl=context,
		)

		yield from self._authenticate_websocket(websocket, event_handler)
		yield from self._start_loop(websocket, event_handler)

	@asyncio.coroutine
	def _start_loop(self, websocket, event_handler):
		"""
		We will listen for websockets events, sending a heartbeat/pong everytime
		we react a TimeoutError. If we don't the webserver would close the idle connection,
		forcing us to reconnect.
		"""
		log.debug('Starting websocket loop')
		while True:
			try:
				yield from asyncio.wait_for(
					self._wait_for_message(websocket, event_handler),
					timeout=self.options['timeout']
				)
			except asyncio.TimeoutError:
				yield from websocket.pong()
				log.debug("Sending heartbeat...")
				continue

	@asyncio.coroutine
	def _authenticate_websocket(self, websocket, event_handler):
		"""
		Sends a authentication challenge over a websocket.
		This is not needed when we just send the cookie we got on login
		when connecting to the websocket.
		"""
		json_data = json.dumps({
			"seq": 1,
			"action": "authentication_challenge",
			"data": {
				"token": self._token
			}
		}).encode('utf8')
		yield from websocket.send(json_data)
		while True:
			message = yield from websocket.recv()
			status = json.loads(message)
			log.debug(status)
			# We want to pass the events to the event_handler already
			# because the hello event could arrive before the authentication ok response
			yield from event_handler(message)
			if ('status' in status and status['status'] == 'OK') and \
					('seq_reply' in status and status['seq_reply'] == 1):
				log.info('Websocket authentification OK')
				return True

	@asyncio.coroutine
	def _wait_for_message(self, websocket, event_handler):
		while True:
			message = yield from websocket.recv()
			yield from event_handler(message)


from .base import Base


class Files(Base):
	endpoint = '/files'

	def upload_file(self, data):
		return self.client.post(
			self.endpoint,
			data=data
		)

	def get_file(self, file_id):
		return self.client.get(
			self.endpoint + '/' + file_id,
		)

	def get_file_thumbnail(self, file_id):
		return self.client.get(
			self.endpoint + '/' + file_id + '/thumbnail',
		)

	def get_file_preview(self, file_id):
		return self.client.get(
			self.endpoint + '/' + file_id + '/preview',
		)

	def get_public_file_link(self, file_id):
		return self.client.get(
			self.endpoint + '/' + file_id + '/link',
		)

	def get_file_metadata(self, file_id):
		return self.client.get(
			self.endpoint + '/' + file_id + '/info',
		)


from .base import Base
from .users import Users


class Preferences(Base):

	def get_user_preferences(self, user_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/preferences'
		)

	def save_user_preferences(self, user_id, options=None):
		return self.client.put(
			Users.endpoint + '/' + user_id + '/preferences',
			options=options
		)

	def delete_user_preferences(self, user_id, options=None):
		return self.client.post(
			Users.endpoint + '/' + user_id + '/preferences/delete',
			options=options
		)

	def list_user_preferences_by_category(self, user_id, category):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/preferences/' + category
		)

	def get_specific_user_preference(self, user_id, category, preference_name):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/preferences/' + category + '/name/' + preference_name
		)


from .base import Base
from .users import Users


class Teams(Base):
	endpoint = '/teams'

	def create_team(self, options=None):
		return self.client.post(
			self.endpoint,
			options
		)

	def get_teams(self, params=None):
		return self.client.get(
			self.endpoint,
			params=params
		)

	def get_team(self, team_id):
		return self.client.get(
			self.endpoint + '/' + team_id,
		)

	def update_team(self, team_id, options=None):
		return self.client.put(
			self.endpoint + '/' + team_id,
			options
		)

	def delete_team(self, team_id, params=None):
		return self.client.delete(
			self.endpoint + '/' + team_id,
			params=params
		)

	def patch_team(self, team_id, options=None):
		return self.client.put(
			self.endpoint + '/' + team_id + '/patch',
			options
		)

	def get_team_by_name(self, name):
		return self.client.get(
			self.endpoint + '/name/' + name
		)

	def search_teams(self, options=None):
		return self.client.post(
			self.endpoint + '/search',
			options
		)

	def check_team_exists(self, name):
		return self.client.get(
			self.endpoint + '/name/' + name + '/exists'
		)

	def get_user_teams(self, user_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams'
		)

	def get_team_members(self, team_id, params=None):
		return self.client.get(
			self.endpoint + '/' + team_id + '/members',
			params=params
		)

	def add_user_to_team(self, team_id, options=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/members',
			options=options
		)

	def add_user_to_team_from_invite(self, params=None):
		return self.client.post(
			self.endpoint + '/members/invite',
			params=params
		)

	def add_multiple_users_to_team(self, team_id, options=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/members/batch',
			options=options
		)

	def get_team_members_for_user(self, user_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams/members'
		)

	def get_team_member(self, team_id, user_id):
		return self.client.get(
			self.endpoint + '/' + team_id + '/members/' + user_id
		)

	def remove_user_from_team(self, team_id, user_id):
		return self.client.delete(
			self.endpoint + '/' + team_id + '/members/' + user_id
		)

	def get_team_members_by_id(self, team_id, options=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/members/ids',
			options
		)

	def get_team_stats(self, team_id):
		return self.client.get(
			self.endpoint + '/' + team_id + '/stats'
		)

	def update_team_member_roles(self, team_id, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + team_id + '/members/' + user_id + '/roles',
			options
		)

	def get_team_unreads_for_user(self, user_id, params=None):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams/unread',
			params=params
		)

	def get_unreads_for_team(self, user_id, team_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams/' + team_id + '/unread',
		)

	def invite_users_to_team_by_mail(self, team_id, options=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/invite/email',
			options
		)

	def import_team_from_other_app(self, team_id, data=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/import',
			data=data
		)

	def get_invite_info_for_team(self, invite_id):
		return self.client.get(
			self.endpoint + '/invite/' + invite_id,
		)

	def get_public_channels(self, team_id, params=None):
		return self.client.get(
			self.endpoint + '/' + team_id + '/channels',
			params=params
		)

	def get_deleted_channels(self, team_id, params=None):
		return self.client.get(
			self.endpoint + '/' + team_id + '/channels/deleted',
			params=params
		)

	def search_channels(self, team_id, options=None):
		return self.client.post(
			self.endpoint + '/' + team_id + '/channels/search',
			options=options
		)


from .base import Base
from .users import Users


class OAuth(Base):
	endpoint = '/oauth'

	def register_oauth_app(self, options):
		return self.client.post(
			self.endpoint + '/apps',
			options=options
		)

	def get_oauth_apps(self, params=None):
		return self.client.get(
			self.endpoint + '/apps',
			params=params
		)

	def get_oauth_app(self, app_id):
		return self.client.get(
			self.endpoint + '/apps/' + app_id
		)

	def delete_oauth_app(self, app_id):
		return self.client.delete(
			self.endpoint + '/apps/' + app_id
		)

	def regenerate_oauth_app_secret(self, app_id):
		return self.client.post(
			self.endpoint + '/apps/' + app_id + '/regen_secret'
		)

	def get_info_on_oauth_app(self, app_id):
		return self.client.get(
			self.endpoint + '/apps/' + app_id + '/info'
		)

	def get_authorized_oauth_apps(self, user_id, params=None):
		return self.client.get(
			Users.endpoint + user_id + '/oauth/apps/authorized',
			params=params
		)


from .base import Base


class System(Base):
	def check_system_health(self):
		return self.client.get(
			'/system/ping'
		)

	def recycle_database_connection(self):
		return self.client.post(
			'/database/recycle'
		)

	def send_test_email(self):
		return self.client.post(
			'/email/test'
		)

	def get_configuration(self):
		return self.client.get(
			'/config'
		)

	def update_configuration(self):
		return self.client.put(
			'/config'
		)

	def reload_configuration(self):
		return self.client.post(
			'/config/reload'
		)

	def get_client_configuration(self, params):
		return self.client.get(
			'/config/client',
			params=params
		)

	def upload_license_file(self, data):
		return self.client.post(
			'/license',
			data=data
		)

	def remove_license_file(self):
		return self.client.delete(
			'/license'
		)

	def get_client_license(self, params):
		return self.client.get(
			'/license/client',
			params=params
		)

	def get_audits(self, params):
		return self.client.get(
			'/audits',
			params=params
		)

	def invalidate_all_caches(self):
		return self.client.post(
			'/caches/invalidate',
		)

	def get_logs(self, params):
		return self.client.get(
			'/logs',
			params=params
		)

	def add_log_message(self, options):
		return self.client.post(
			'/logs',
			options=options
		)

	def get_webrtc_token(self):
		return self.client.get(
			'/webrtc/token'
		)

	def get_analytics(self, params):
		return self.client.get(
			'/analytics/old',
			params=params
		)



from requests import HTTPError


class InvalidOrMissingParameters(HTTPError):
	"""
	Raised when mattermost returns a
	400 Invalid or missing parameters in URL or request body
	"""


class NoAccessTokenProvided(HTTPError):
	"""
	Raised when mattermost returns a
	401 No access token provided
	"""


class NotEnoughPermissions(HTTPError):
	"""
	Raised when mattermost returns a
	403 Do not have appropriate permissions
	"""


class ResourceNotFound(HTTPError):
	"""
	Raised when mattermost returns a
	404 Resource not found
	"""


class ContentTooLarge(HTTPError):
	"""
	Raised when mattermost returns a
	413 Content too large
	"""


class FeatureDisabled(HTTPError):
	"""
	Raised when mattermost returns a
	501 Feature is disabled
	"""



from .base import Base


class Emoji(Base):
	endpoint = '/emoji'

	def create_custom_emoji(self, data):
		return self.client.post(
			self.endpoint,
			data=data
		)

	def get_emoji_list(self, params=None):
		return self.client.get(
			self.endpoint,
			params=params
		)

	def get_custom_emoji(self, emoji_id):
		return self.client.get(
			self.endpoint + '/' + emoji_id
		)

	def delete_custom_emoji(self, emoji_id):
		return self.client.delete(
			self.endpoint + '/' + emoji_id
		)

	def get_custom_emoji_image(self, emoji_id):
		return self.client.get(
			self.endpoint + '/' + emoji_id + '/image'
		)



from .base import Base


class Webhooks(Base):
	endpoint = '/hooks'

	def create_incoming_hook(self, options):
		return self.client.post(
			self.endpoint + '/incoming',
			options=options
		)

	def list_incoming_hooks(self, params):
		return self.client.get(
			self.endpoint + '/incoming',
			params=params
		)

	def get_incoming_hook(self, hook_id):
		return self.client.get(
			self.endpoint + '/incoming/' + hook_id
		)

	def update_incoming_hook(self, hook_id, options):
		return self.client.put(
			self.endpoint + '/incoming/' + hook_id,
			options=options
		)

	def create_outgoing_hook(self, options):
		return self.client.post(
			self.endpoint + '/outgoing',
			options=options
		)

	def list_outgoing_hooks(self, params):
		return self.client.get(
			self.endpoint + '/outgoing',
			params=params
		)

	def get_outgoing_hook(self, hook_id):
		return self.client.get(
			self.endpoint + '/outgoing/' + hook_id
		)

	def delete_outgoing_hook(self, hook_id):
		return self.client.delete(
			self.endpoint + '/outgoing/' + hook_id
		)

	def update_outgoing_hook(self, hook_id, options):
		return self.client.put(
			self.endpoint + '/outgoing/' + hook_id,
			options=options
		)

	def regenerate_token_outgoing_hook(self, hook_id):
		return self.client.put(
			self.endpoint + '/outgoing/' + hook_id + '/regen_token'
		)


from .base import Base
from .teams import Teams


class Commands(Base):
	endpoint = '/commands'

	def create_command(self, options=None):
		return self.client.post(
			self.endpoint,
			options=options
		)

	def list_commands_for_team(self, params=None):
		return self.client.get(
			self.endpoint,
			params=params
		)

	def list_autocomplete_commands(self, team_id):
		return self.client.get(
			Teams.endpoint + team_id + '/commands/autocomplete'
		)

	def update_command(self, command_id, options=None):
		return self.client.put(
			self.endpoint + command_id,
			options=options
		)

	def delete_command(self, command_id):
		return self.client.delete(
			self.endpoint + command_id
		)

	def generate_new_token(self, command_id):
		return self.client.put(
			self.endpoint + command_id + '/regen_token'
		)

	def execute_command(self, options=None):
		return self.client.post(
			self.endpoint + '/execute',
			options=options
		)


import json

import logging
import requests

from .exceptions import (
	InvalidOrMissingParameters,
	NoAccessTokenProvided,
	NotEnoughPermissions,
	ContentTooLarge,
	FeatureDisabled
)

logging.basicConfig(level=logging.INFO)
log = logging.getLogger('mattermostdriver.websocket')


class Client:
	def __init__(self, options):
		self._url = '{scheme:s}://{url:s}{basepath:s}'.format(
			scheme=options['scheme'],
			url=options['url'],
			basepath=options['basepath']
		)
		self._scheme = options['scheme']
		self._basepath = options['basepath']
		self._port = options['port']
		self._verify = options['verify']
		self._token = ''
		self._cookies = None
		self._userid = ''
		self._username = ''

	@property
	def userid(self):
		"""
		:return: The user id of the logged in user
		"""
		return self._userid

	@userid.setter
	def userid(self, user_id):
		self._userid = user_id

	@property
	def username(self):
		"""
		:return: The username of the logged in user. If none, returns an emtpy string.
		"""
		return self._username

	@username.setter
	def username(self, username):
		self._username = username

	@property
	def url(self):
		return self._url

	@property
	def cookies(self):
		"""
		:return: The cookie given on login
		"""
		return self._cookies

	@cookies.setter
	def cookies(self, cookies):
		self._cookies = cookies

	@property
	def token(self):
		"""
		:return: The token for the login
		"""
		return self._token

	@token.setter
	def token(self, t):
		self._token = t

	def auth_header(self):
		if self._token == '':
			return {}
		return {"Authorization": "Bearer {token:s}".format(token=self._token)}

	def make_request(self, method, endpoint, options=None, params=None, data=None):
		if options is None:
			options = {}
		if params is None:
			params = {}
		if data is None:
			data = {}
		method = method.lower()
		request = requests.get
		if method == 'post':
			request = requests.post
		elif method == 'put':
			request = requests.put
		elif method == 'delete':
			request = requests.delete

		response = request(
				self.url + endpoint,
				headers=self.auth_header(),
				verify=self._verify,
				json=options,
				params=params,
				data=data
			)
		try:
			response.raise_for_status()
		except requests.HTTPError as e:
			data = json.loads(e.response.text)
			if data['status_code'] == 400:
				raise InvalidOrMissingParameters(data['message'])
			elif data['status_code'] == 401:
				raise NoAccessTokenProvided(data['message'])
			elif data['status_code'] == 403:
				raise NotEnoughPermissions(data['message'])
			elif data['status_code'] == 413:
				raise ContentTooLarge(data['message'])
			elif data['status_code'] == 501:
				raise FeatureDisabled(data['message'])
			else:
				raise

		log.debug(json.loads(response.text))
		return response

	def get(self, endpoint, options=None, params=None):
		return json.loads(
			self.make_request('get', endpoint, options=options, params=params).text
		)

	def post(self, endpoint, options=None, params=None, data=None):
		return json.loads(
			self.make_request('post', endpoint, options=options, params=params, data=data).text
		)

	def put(self, endpoint, options=None, params=None, data=None):
		return json.loads(
			self.make_request('put', endpoint, options=options, params=params, data=data).text
		)

	def delete(self, endpoint, options=None, params=None, data=None):
		return json.loads(
			self.make_request('delete', endpoint, options=options, params=params, data=data).text
		)


from .base import Base
from .teams import Teams
from .users import Users
from .channels import Channels


class Posts(Base):
	endpoint = '/posts'

	def create_post(self, options):
		return self.client.post(
			self.endpoint,
			options=options
		)

	def get_post(self, post_id):
		return self.client.get(
			self.endpoint + '/' + post_id,
		)

	def delete_post(self, post_id):
		return self.client.delete(
			self.endpoint + '/' + post_id,
		)

	def update_post(self, post_id, options=None):
		return self.client.put(
			self.endpoint + '/' + post_id,
			options=options
		)

	def patch_post(self, post_id, options=None):
		return self.client.put(
			self.endpoint + '/' + post_id + '/patch',
			options=options
		)

	def get_thread(self, post_id):
		return self.client.get(
			self.endpoint + '/' + post_id + '/thread',
		)

	def get_list_of_flagged_posts(self, user_id, params=None):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/posts/flagged',
			params=params
		)

	def get_file_info_for_post(self, post_id):
		return self.client.get(
			self.endpoint + '/' + post_id + '/files/info',
		)

	def get_posts_for_channel(self, channel_id, params=None):
		return self.client.get(
			Channels.endpoint + '/' + channel_id + '/posts',
			params=params
		)

	def search_for_team_posts(self, team_id, options):
		return self.client.post(
			Teams.endpoint + '/' + team_id + '/posts/search',
			options=options
		)

	def pin_post_to_channel(self, post_id):
		return self.client.post(
			self.endpoint + '/' + post_id + '/pin'
		)

	def unpin_post_to_channel(self, post_id):
		return self.client.post(
			self.endpoint + '/' + post_id + '/unpin'
		)


import asyncio
import logging

from src.mattermostdriver.endpoints.elasticsearch import Elasticsearch
from src.mattermostdriver.endpoints.emoji import Emoji
from .client import Client
from .websocket import Websocket
from .endpoints.brand import Brand
from .endpoints.channels import Channels
from .endpoints.cluster import Cluster
from .endpoints.commands import Commands
from .endpoints.compliance import Compliance
from .endpoints.files import Files
from .endpoints.ldap import LDAP
from .endpoints.oauth import OAuth
from .endpoints.posts import Posts
from .endpoints.preferences import Preferences
from .endpoints.saml import SAML
from .endpoints.system import System
from .endpoints.teams import Teams
from .endpoints.users import Users
from .endpoints.webhooks import Webhooks

logging.basicConfig(level=logging.INFO)
log = logging.getLogger('mattermostdriver.api')


class Driver:
	"""
	Contains the client, api and provides you with functions for
	login, logout and initializing a websocket connection.
	"""

	default_options = {
		'scheme': 'https',
		'url': 'localhost',
		'port': 8065,
		'basepath': '/api/v4',
		'verify': True,
		'timeout': 30,
		'login_id': None,
		'password': None,
	}
	"""
	Required:
		- login_id
		- password
	
	Optional:
		- scheme
		- url (though it would be a good idea to change that)
		- port
		- verify
		- timeout
	
	Should not be changed:
		- basepath - unlikeliy this would do any good
	"""

	def __init__(self, options=default_options):
		"""
		:param options: A dict with the values from `default_options`
		:type options: dict
		"""
		if options is None:
			options = self.default_options
		self.options = self.default_options.copy()
		self.options.update(options)
		self.driver = self.options
		self.client = Client(self.options)
		self.api = {
			'users': Users(self.client),
			'teams': Teams(self.client),
			'channels': Channels(self.client),
			'posts': Posts(self.client),
			'files': Files(self.client),
			'preferences': Preferences(self.client),
			'emoji': Emoji(self.client),
			'system': System(self.client),
			'webhooks': Webhooks(self.client),
			'commands': Commands(self.client),
			'compliance': Compliance(self.client),
			'cluster': Cluster(self.client),
			'brand': Brand(self.client),
			'oauth': OAuth(self.client),
			'saml': SAML(self.client),
			'ldap': LDAP(self.client),
			'elasticsearch': Elasticsearch(self.client),
		}
		self.websocket = None

	def init_websocket(self, event_handler):
		"""
		Will initialize the websocket connection to the mattermost server.

		This should be run after login(), because the websocket needs to make
		an authentification.

		See https://api.mattermost.com/v4/#tag/WebSocket for which
		websocket events mattermost sends.

		:param event_handler: The function to handle the websocket events.
		:type event_handler: Function
		:return: The event loop
		"""
		self.websocket = Websocket(self.options, self.client.token)
		loop = asyncio.get_event_loop()
		loop.run_until_complete(self.websocket.connect(event_handler))
		return loop

	def login(self):
		"""
		Logs the user in.

		The log in information is saved in the client:

		- userid
		- username
		- cookies

		:return: The raw response from the request
		"""
		result = self.api['users'].login_user({
			'login_id': self.options['login_id'],
			'password': self.options['password'],
		})
		log.debug(result)
		if result.status_code == 200:
			self.client.token = result.headers['Token']
			self.client.cookies = result.cookies
			if 'id' in result:
				self.client.userid = result['id']
			if 'username' in result:
				self.client.username = result['username']
		return result

	def logout(self):
		"""
		Log the user out.

		:return: The JSON response from the server
		"""
		self.client.token = ''
		self.client.userid = ''
		self.client.username = ''
		self.client.cookies = None
		return self.api['users'].logout_user()



from .base import Base


class LDAP(Base):
	endpoint = '/ldap'

	def sync_ldap(self):
		return self.client.post(
			self.endpoint + '/sync'
		)

	def test_ldap_config(self):
		return self.client.post(
			self.endpoint + '/test'
		)



from .base import Base
from .teams import Teams
from .users import Users


class Channels(Base):
	endpoint = '/channels'

	def create_channel(self, options):
		return self.client.post(
			self.endpoint,
			options=options
		)

	def create_direct_message_channel(self, options):
		return self.client.post(
			self.endpoint + '/direct',
			options=options
		)

	def create_group_message_channel(self, options):
		return self.client.post(
			self.endpoint + '/group',
			options=options
		)

	def get_list_of_channels_by_ids(self, team_id, options=None):
		return self.client.post(
			Teams.endpoint + '/' + team_id + '/channels/ids',
			options=options
		)

	def get_channel(self, channel_id):
		return self.client.get(
			self.endpoint + '/' + channel_id
		)

	def update_channel(self, channel_id, options):
		return self.client.put(
			self.endpoint + '/' + channel_id,
			options=options
		)

	def delete_channel(self, channel_id):
		return self.client.delete(
			self.endpoint + '/' + channel_id
		)

	def patch_channel(self, channel_id, options):
		return self.client.put(
			self.endpoint + '/' + channel_id + '/patch',
			options=options
		)

	def restore_channel(self, channel_id):
		return self.client.post(
			self.endpoint + '/' + channel_id + '/restore',
		)

	def get_channel_statistics(self, channel_id):
		return self.client.get(
			self.endpoint + '/' + channel_id + '/stats',
		)

	def get_channel_pinned_posts(self, channel_id):
		return self.client.get(
			self.endpoint + '/' + channel_id + '/pinned'
		)

	def get_channel_by_name(self, team_id, channel_name):
		return self.client.get(
			Teams.endpoint + '/' + team_id + '/channels/name/' + channel_name
		)

	def get_channel_by_name_and_team_name(self, team_name, channel_name):
		return self.client.get(
			Teams.endpoint + '/name/' + team_name + '/channels/name/' + channel_name
		)

	def get_channel_members(self, channel_id, params=None):
		return self.client.get(
			self.endpoint + '/' + channel_id + '/members',
			params=params
		)

	def add_user(self, channel_id, options=None):
		return self.client.post(
			self.endpoint + '/' + channel_id + '/members',
			options=options
		)

	def get_channel_members_by_ids(self, channel_id, options=None):
		return self.client.post(
			self.endpoint + '/' + channel_id + '/members/ids',
			options=options
		)

	def get_channel_member(self, channel_id, user_id):
		return self.client.get(
			self.endpoint + '/' + channel_id + '/members/' + user_id
		)

	def remove_channel_member(self, channel_id, user_id):
		return self.client.delete(
			self.endpoint + '/' + channel_id + '/members/' + user_id
		)

	def update_channel_roles(self, channel_id, user_id, options):
		return self.client.put(
			self.endpoint + '/' + channel_id + '/members/' + user_id + '/roles',
			options=options
		)

	def update_channel_notifications(self, channel_id, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + channel_id + '/members/' + user_id + '/notify_props',
			options=options
		)

	def view_channel(self, user_id, options):
		return self.client.post(
			self.endpoint + '/members/' + user_id + '/view',
			options=options
		)

	def get_channel_members_for_user(self, user_id, team_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams/' + team_id + '/channels/members'
		)

	def get_channel_for_user(self, user_id, team_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/teams/' + team_id + '/channels'
		)

	def get_unread_messages(self, user_id, channel_id):
		return self.client.get(
			Users.endpoint + '/' + user_id + '/channels/' + channel_id + '/unread'
		)



from .base import Base


class Compliance(Base):
	endpoint = '/compliance'

	def create_report(self):
		return self.client.post(
			self.endpoint + '/reports'
		)

	def get_reports(self, params=None):
		return self.client.get(
			self.endpoint + '/reports',
			params=params
		)

	def get_report(self, report_id):
		return self.client.get(
			self.endpoint + '/reports/' + report_id
		)

	def download_report(self, report_id):
		return self.client.get(
			self.endpoint + '/reports/' + report_id + '/download'
		)


from .base import Base


class Users(Base):
	endpoint = '/users'

	def login_user(self, options):
		return self.client.make_request('post', self.endpoint + '/login', options)

	def logout_user(self):
		return self.client.post(self.endpoint + '/logout')

	def create_user(self, options=None):
		return self.client.post(self.endpoint, options)

	def get_users(self, params=None):
		return self.client.get(	self.endpoint, params=params)

	def get_users_by_ids(self, options=None):
		return self.client.post(
			self.endpoint + '/ids',
			options
		)

	def get_users_by_usernames(self, options=None):
		return self.client.get(
			self.endpoint + '/usernames',
			options
		)

	def search_users(self, options=None):
		return self.client.get(
			self.endpoint + '/search',
			options
		)

	def autocomplete_users(self, params=None):
		return self.client.get(
			self.endpoint + '/autocomplete',
			params=params
		)

	def get_user(self, user_id):
		return self.client.get(
			self.endpoint + '/' + user_id
		)

	def update_user(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id,
			options
		)

	def deactivate_user(self, user_id):
		return self.client.delete(
			self.endpoint + '/' + user_id
		)

	def patch_user(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id + '/patch',
			options
		)

	def update_user_role(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id + '/roles',
			options
		)

	def update_user_active_status(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id + '/active',
			options
		)

	def get_user_profile_image(self, user_id):
		return self.client.get(
			self.endpoint + '/' + user_id + '/image'
		)

	def set_user_profile_image(self, user_id, data=None):
		return self.client.post(
			self.endpoint + '/' + user_id + '/image',
			data=data
		)

	def get_user_by_username(self, username):
		return self.client.get(
			self.endpoint + '/username/' + username
		)

	def reset_password(self, options=None):
		return self.client.post(
			self.endpoint + '/password/reset',
			options
		)

	def update_user_mfa(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id + '/mfa',
			options
		)

	def generate_mfa(self, user_id):
		return self.client.post(
			self.endpoint + '/' + user_id + '/mfa/generate'
		)

	def check_mfa(self, options=None):
		return self.client.post(
			self.endpoint + '/mfa',
			options
		)

	def update_user_password(self, user_id, options=None):
		return self.client.put(
			self.endpoint + '/' + user_id + '/password',
			options
		)

	def send_password_reset_mail(self, options=None):
		return self.client.post(
			self.endpoint + '/password/reset/send',
			options
		)

	def get_user_by_email(self, email):
		return self.client.post(
			self.endpoint + '/email/' + email
		)

	def get_user_sessions(self, user_id):
		return self.client.get(
			self.endpoint + '/' + user_id + '/sessions'
		)

	def revoke_user_session(self, user_id, options=None):
		return self.client.post(
			self.endpoint + '/' + user_id + '/sessions/revoke',
			options
		)

	def attach_mobile_device(self, options=None):
		return self.client.put(
			self.endpoint + '/sessions/device',
			options
		)

	def get_user_audits(self, user_id):
		return self.client.get(
			self.endpoint + '/' + user_id + '/audits'
		)

	def verify_user_email(self, options=None):
		return self.client.post(
			self.endpoint + '/email/verify',
			options
		)

	def send_verification_mail(self, options=None):
		return self.client.post(
			self.endpoint + '/email/verify/send',
			options
		)

	def switch_login_method(self, options=None):
		return self.client.post(
			self.endpoint + '/login/switch',
			options
		)



import os.path
import sys

from setuptools import setup, find_packages

full_version = ''

root_dir = os.path.abspath(os.path.dirname(__file__))

readme_file = os.path.join(root_dir, 'README.rst')
with open(readme_file, encoding='utf-8') as f:
	long_description = f.read()

version_module = os.path.join(root_dir, 'src', 'mattermostdriver', 'version.py')
with open(version_module, encoding='utf-8') as f:
	exec(f.read())

py_version = sys.version_info[:2]

if py_version < (3, 3):
	raise Exception("python-mattermost-driver requires Python >= 3.3.")

setup(
	name='mattermostdriver',
	version=full_version,
	description='A Python Mattermost Driver',
	long_description=long_description,
	url='https://github.com/Vaelor/python-mattermost-driver',
	author='Christian Plümer',
	author_email='github@kuuku.net',
	license='MIT',
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Environment :: Web Environment',
		'Intended Audience :: Developers',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
	],
	package_dir={'': 'src'},
	packages=find_packages('src'),
	extras_require={
		':python_version=="3.3"': ['asyncio'],
	},
	install_requires=[
		'websockets>=3.2',
		'requests>=2.1'
	],
)



from .base import Base


class SAML(Base):
	endpoint = '/saml'

	def get_metadata(self):
		return self.client.get(
			self.endpoint + '/metadata'
		)

	def upload_idp_certificate(self, data):
		return self.client.post(
			self.endpoint + '/certificate/idp',
			data=data
		)

	def remove_idp_certificate(self):
		return self.client.delete(
			self.endpoint + '/certificate/idp'
		)

	def upload_public_certificate(self, data):
		return self.client.post(
			self.endpoint + '/certificate/public',
			data=data
		)

	def remove_public_certificate(self):
		return self.client.delete(
			self.endpoint + '/certificate/public'
		)

	def upload_private_key(self, data):
		return self.client.post(
			self.endpoint + '/certificate/private',
			data=data
		)

	def remove_private_key(self):
		return self.client.delete(
			self.endpoint + '/certificate/private'
		)

	def get_certificate_status(self):
		return self.client.get(
			self.endpoint + '/certificate/status'
		)